<?xml version="1.0" encoding="UTF-8" ?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">

<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">

<head>
<title>{package} version {version} run-time statistics</title>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />

<style type="text/css" media="screen">
<!--/*--><![CDATA[<!--*/

th, td
{
  text-align: left;
  padding: 0.5em;
  border: 1px solid gray;
}

/*]]>*/-->
</style>

</head>

<body>

<h1>{package} version {version} run-time statistics</h1>

<table>

<tr>
  <th>Name</th>
  <th>Value</th>
</tr>

<tr>
  <td>Number of open connections</td>
  <td>{opens}</td>
</tr>

<tr>
  <td>Number of requests</td>
  <td>{reqs}</td>
</tr>

<tr>
  <td>Number of bad connections</td>
  <td>{badconns}</td>
</tr>

<tr>
  <td>Number of denied connections</td>
  <td>{deniedconns}</td>
</tr>

<tr>
  <td>Number of refused connections due to high load</td>
  <td>{refusedconns}</td>
</tr>

</table>

<hr />

<p><em>Generated by <a href="{website}">{package}</a> version {version}.</em></p>

</body>

</html>
